#!/bin/bash
#Description: lib check
#Copyright: Copyright © Huawei Technologies Co., Ltd. 2025. All rights reserved.
#History: 2023-06-19 created

current_dir=$(cd $(dirname $0); pwd)
source "${current_dir}/const.sh"
source "${current_dir}/utils.sh"

INSTALL_LOG="${current_dir}/sys_diagnosis_install.log"
TEMP_INI="${current_dir}/sys_diagnosis_env_res.ini"

ESSENTIAL_LOST="essential_lost"
NOESSENTIAL_LOST="noessential_lost"

# 环境检查默认结果
ENV_RESULT="${SUCCESS}"

need_install_lib=()
os_lib_list=()
pre_lib_list=()
essential_lib_list=()
non_essential_lib_list=()
lib_path=()


# 版本比较：version_lt [result_version] [standard_version]
function version_lt()
{
    test "$(echo "$@" | tr " " "\n" | sort -rV | head -n 1)" != "$1";
}

function init_variable() {
    # 升级场景下，使用安装路径下的log.sh
    local log_sh="${INSTALL_PATH}/tools/log.sh"
    if [[ ! -f "${log_sh}" ]]; then
        # 跟随框架安装的场景下，安装包中的log.sh路径
        log_sh="${current_dir}/../../../log.sh"
        if [[ ! -f "${log_sh}" ]]; then
            # 通过节点安装时，安装路径中的log.sh路径
            log_sh="${current_dir}/../../../tools/log.sh"
            if [[ ! -f "${log_sh}" ]]; then
                # 通过页面安装插件时，插件解压路径的相对路径
                log_sh="${current_dir}/../../../../tools/log.sh"
                if [[ ! -f "${log_sh}" ]]; then
                    echo "There is no log utils(log.sh), please check!"
                    exit "${ERROR}"
                else
                    source ${log_sh}
                fi
            else
                source ${log_sh}
            fi
        else
            source ${log_sh}
        fi
    else
        source ${log_sh}
    fi
    if [[ -f "${INSTALL_PATH}/logs/sys_diagnosis_install.log" ]]; then
        INSTALL_LOG="${INSTALL_PATH}/logs/sys_diagnosis_install.log"
    fi
    if [[ ! -f "${INSTALL_LOG}" ]]; then
        touch "${INSTALL_LOG}"
        chmod 600 "${INSTALL_LOG}"
    fi
    # 每次检查前先删掉该临时文件
    if [[ -f "${TEMP_INI}" ]]; then
        rm -f "${TEMP_INI}"
    fi
    local config_file="${CONFIG_FILE}"
    if [[ ! -f "${config_file}" ]]; then
        config_file="${current_dir}/../../../install.ini"
    fi
    if [[ ! -f "${config_file}" ]]; then
        echo_and_log "There is no install.ini, please check!" "${ERROR}" "${INSTALL_LOG}"
        exit "${ERROR}"
    fi
    os_type=$(grep -w OS_TYPE "${config_file}" | awk -F= '{if($2 !="") print $2}')
    os_version=$(grep -w OS_VERSION "${config_file}" | awk -F= '{if($2 !="") print $2}')
}

function choose_installer() {
    installer_tool=${COMMAND_LIST["${os_type}"]}
    if [[ -z "${installer_tool}" ]]; then
        echo_and_log "Can't found package tool for for this os: ${os_type}" "${ERROR}" "${INSTALL_LOG}"
        for ((i=0;i<${#PACKAGE_TOOLS_LIST[@]};i++))
        do
            if which ${PACKAGE_TOOLS_LIST[$i]} >> /dev/null 2>&1; then
                installer_tool=${PACKAGE_TOOLS_LIST[$i]}
            fi
        done

    fi
    echo "package_tool=${installer_tool}" >> "${TEMP_INI}"
    echo_and_log "Use ${installer_tool} package tool" "${SUCCESS}" "${INSTALL_LOG}"
}

# 检查磁盘空间是否足够安装
function check_install_space() {
    # 磁盘空间大小检测
    echo_and_log "Check disk space" "${DOING}" "${INSTALL_LOG}"
    disk_space_check "${INSTALL_PATH}" "${INSTALL_SPACE_NEED}"
    if [[ "$?" -ne "${SUCCESS}" ]]; then
        echo "env_disk_lack=${print_disk_space_need}" >> "${TEMP_INI}"
        ENV_RESULT="${ERROR}"
        echo_and_log "Check install path disk space Failed" "${ERROR}" "${INSTALL_LOG}"
    else
        echo_and_log "Check install path disk space Success" "${SUCCESS}" "${INSTALL_LOG}"
    fi
}

function check_os_type()
{
    echo_and_log "os_type: ${os_type}" "${SUCCESS}" "${INSTALL_LOG}"
    echo_and_log "os_version: ${os_version}" "${SUCCESS}" "${INSTALL_LOG}"
    local support_flag=1
    if [[ -n "${os_type}" ]]; then
        for ((i=0;i<${#OS_TYPE_LIST[@]};i++))
        do
            if [[ "${os_type}" == "${OS_TYPE_LIST[$i]}" ]]; then
                support_flag=0
                break
            fi
        done
    fi
    if [[ "${support_flag}" -eq 1 ]]; then
        local old_os_type="${os_type}"
        os_type="${OS_MAP[${os_type}]}"
        if [[ -z "${os_type}" ]]; then
            echo "env_os_supported=false" >> "${TEMP_INI}"
            echo_and_log "Unsupported OS type: ${old_os_type}" "${ERROR}" "${INSTALL_LOG}"
            exit "${ERROR}"
        else
            echo_and_log "${os_type} is selected as the compatible OS based on ${old_os_type} affinity." "${SUCCESS}" "${INSTALL_LOG}"
        fi
    fi
}

function choose_list()
{
    for element in ${package_all[@]}
    do
        if [[ "${element}" == "pre" ]]; then
           pre_lib_list=(${PRE_LIB_LIST["${os_type}"]})
        elif [[ "${element}" == "essential" ]]; then
            essential_lib_list=(${ESSENTIAL_LIB_LIST["${os_type}"]})
        elif [[ "${element}" == "non_essential" ]]; then
            non_essential_lib_list=(${NON_ESSENTIAL_LIB_LIST["${os_type}"]})
        else
            echo_and_log "Install type error: pre，essential or non_essential." "${ERROR}" "${INSTALL_LOG}"
            exit "${ERROR}"
        fi
    done
}

function cmd_exec()
{
    ${temp_arr["$1"]} >> /dev/null 2>&1
    if [[ "$?" -eq "${SUCCESS}" ]]; then
        echo_and_log "The ${1}(${temp_arr["$1"]})  tool cmd check: OK" "${SUCCESS}" "${INSTALL_LOG}"
    else
        echo_and_log "The ${1}(${temp_arr["$1"]}) tool cmd check: Failed" "${ERROR}" "${INSTALL_LOG}"
        need_install_lib[${#need_install_lib[@]}]=$1
    fi
}

#检查依赖包kernel-debuginfo
function  check_kernel_debuginfo()
{
    if [[ "${installer_tool}" == "yum" ]]; then
        rpm -qa | grep "kernel-debuginfo" > /dev/null
        if [[ "$?" -ne "${SUCCESS}" ]]; then
            echo_and_log "Kernel-debuginfo may not be installed, which may cause some kernel symbol information not found" "${DOING}" "${INSTALL_LOG}"
            non_essential_need_install_lib=(${non_essential_need_install_lib[@]} kernel-debuginfo)
        fi
    elif [[ "${os_type}" == "SLESOS" ]]; then
        rpm -qa | grep "kernel-default-debuginfo" > /dev/null
        if [[ "$?" -ne "${SUCCESS}" ]]; then
            echo_and_log "Kernel-default-debuginfo may not be installed, which may cause some kernel symbol information not found" "${DOING}" "${INSTALL_LOG}"
            non_essential_need_install_lib=(${non_essential_need_install_lib[@]} kernel-default-debuginfo)
        fi
    else
        local flag=$(dpkg-query -l | grep "linux-image-$(uname -r)-dbgsym" | awk 'NR==1{print $1}') >> /dev/null 2>&1
        if [[ "${flag}" != "ii" ]]; then
            echo_and_log "Linux-image-$(uname -r)-dbgsym may not be installed, which may cause some kernel symbol information not found" "${DOING}" "${INSTALL_LOG}"
            non_essential_need_install_lib=(${non_essential_need_install_lib[@]} Linux-image-$(uname -r)-dbgsym)
        fi
    fi
}

# 命令行检测依赖包
function cmd_check()
{
    if [[ "$1" == "glibc" ]]; then
        ldd --version >> /dev/null 2>&1
        if [[ "$?" -eq "${SUCCESS}" ]]; then
            local ldd_ver
            ldd_ver=$(ldd --version)
            sub_string=${ldd_ver#*)}
            ldd_ver=$(echo $sub_string | awk -F# '{print $1,$2}' | awk '{print $1}')
            standard='2.17'
            if version_lt $ldd_ver $standard; then
                echo_and_log "The installed glibc version is lower than the requested, please upgrade glibc to 2.17." "${ERROR}" "${INSTALL_LOG}"
                echo_and_log "Please follow the FAQ in 'Kunpeng Hyper Tuner User Guide' to fix the issue." "${ERROR}" "${INSTALL_LOG}"
                echo "${ESSENTIAL_LOST}=$1" >> "${TEMP_INI}"
            else
                echo_and_log "The installed glibc version check: OK" "${SUCCESS}" "${INSTALL_LOG}"
            fi
        else
            need_install_lib[${#need_install_lib[@]}]=$1
        fi
    elif [[ "$1" == "openssl" ]]; then
        # 使用框架自带的openssl，默认ok
        echo_and_log "The installed openssl version check: OK" "${SUCCESS}" "${INSTALL_LOG}"
    else
        cmd_exec $1
    fi
}

function package_check()
{
    local package_installed=""
    if [[ "${installer_tool}" == "yum" ]] || [[ "${installer_tool}" == "zypper" ]]; then
        package_installed=$(rpm -qa $1)
    else
        package_installed=$(dpkg -l | grep "^ii" | awk '{print $2}' | cut -d ':' -f 1 | grep "^$1")
    fi
    if [[ -n "${package_installed}" ]]; then
        if [[ "$1" == "libffi-devel" ]]; then
            local result
            result=$(rpm -qa libffi-devel | awk -F"-" '{print $1,$2, $3,$4}' | awk '{print $3}')
            standard='3.0.13'
            if version_lt $result $standard; then
                echo_and_log "The installed libffi-devel version check is lower than the requested, please upgrade libffi-devel to 3.0.13." "${ERROR}" "${INSTALL_LOG}"
                echo "${ESSENTIAL_LOST}=$1" >> "${TEMP_INI}"
            else
                echo_and_log "The installed libffi-devel version check: OK" "${SUCCESS}" "${INSTALL_LOG}"
            fi
        elif [[ "$1" == "crontabs" ]] || [[ "$1" == "cron" ]]; then
            if [[ "${installer_tool}" == "yum" ]]; then
                which crond >> /dev/null 2>&1
                if [[ "$?" -eq "${SUCCESS}" ]]; then
                    echo_and_log "The crontabs cmd check: OK" "${SUCCESS}" "${INSTALL_LOG}"
                else
                    need_install_lib[${#need_install_lib[@]}]=$1
                fi
            else
                which cron >> /dev/null 2>&1
                if [[ "$?" -eq "${SUCCESS}" ]]; then
                    echo_and_log "The cron cmd check: OK" "${SUCCESS}" "${INSTALL_LOG}"
                else
                    need_install_lib[${#need_install_lib[@]}]=$1
                fi
            fi
        else
            echo_and_log "The $1 tool package check: OK" "${SUCCESS}" "${INSTALL_LOG}"
        fi
    else
        # 检测源码安装依赖包
        local lib_name_path=${LIB_CHECK_LIST["$1"]}
        if [[ -n "${lib_name_path}" ]]; then
            local check_result=""
            get_lib_path
            for lib in ${lib_path[@]}
            do
                local res
                res=$(find "${lib}" -name "${lib_name_path}" 2>>/dev/null)
                if [[ -n "${res}" ]]; then
                    check_result="0"
                    echo_and_log "The $1 tool package check: OK" "${SUCCESS}" "${INSTALL_LOG}"
                    break
                fi
            done
            if [[ -z "${check_result}" ]]; then
                need_install_lib[${#need_install_lib[@]}]=$1
            fi
        else
            need_install_lib[${#need_install_lib[@]}]=$1
        fi
    fi
}

# 获取需要安装依赖包
function cmd_package_check()
{
    local os_lib_list=($1)
    declare -A temp_arr
    local agent_skip_list=(${AGENT_SKIP_LIST["${os_type}"]})
    if [[ "$2" == "pre_cmd_list" ]]; then
        for key in "${!PRE_CMD_LIST[@]}"
        do
          temp_arr["$key"]="${PRE_CMD_LIST[$key]}"
        done
    elif [[ "$2" == "essential_cmd_list" ]]; then
        for key in "${!ESSENTIAL_CMD_LIST[@]}"
        do
          temp_arr["$key"]="${ESSENTIAL_CMD_LIST[$key]}"
        done
    elif [[ "$2" == "non_essential_cmd_list" ]]; then
        for key in "${!NON_ESSENTIAL_CMD_LIST[@]}"
        do
          temp_arr["$key"]="${NON_ESSENTIAL_CMD_LIST[$key]}"
        done
    fi
    for ((i=0;i<${#os_lib_list[@]};i++))
    do
        if [[ "${install_mode}" == "agent" ]]; then
            if [[ "${agent_skip_list[@]}" =~ "${os_lib_list[$i]}" ]]; then
                continue
            fi
        fi
        if [[ "${temp_arr["${os_lib_list[$i]}"]}" != "package_type" ]]; then
            # 命令行检测
            cmd_check ${os_lib_list[$i]} ${temp_arr["${os_lib_list[$i]}"]}
        elif [[ "${os_lib_list[$i]}" == "linux-tools-version" ]]; then
            package_check "linux-tools-$(uname -r)"
        else
            # 包检测
            package_check ${os_lib_list[$i]}
        fi
    done
    local_need_install_lib=(${need_install_lib[@]})
    need_install_lib=()
}

function lib_check()
{
    echo_and_log "Checking install Dependent Packages" "${DOING}" "${INSTALL_LOG}"
    # 依据os类型进行依赖包检测
    choose_installer
    local i=0
    choose_list

    cmd_package_check "${pre_lib_list[*]}" "pre_cmd_list"
    pre_need_install_lib=(${local_need_install_lib[@]})

    cmd_package_check "${essential_lib_list[*]}" "essential_cmd_list"
    essential_need_install_lib=(${local_need_install_lib[@]})

    cmd_package_check "${non_essential_lib_list[*]}" "non_essential_cmd_list"
    non_essential_need_install_lib=(${local_need_install_lib[@]})

    #检查依赖包kernel-debuginfo
    check_kernel_debuginfo

    all_need_checked_lib=(${pre_need_install_lib[@]} ${essential_need_install_lib[@]} ${non_essential_need_install_lib[@]})
    essential_need_install_lib_print=(${pre_need_install_lib[@]} ${essential_need_install_lib[@]})

    # 非必需依赖只检查，不安装
    if [[ ${#all_need_checked_lib[@]} -eq 0 ]]; then
        echo_and_log "lib check success" "${SUCCESS}" "${INSTALL_LOG}"
        exit "${ENV_RESULT}"
    else
        if [[ ${#essential_need_install_lib_print[@]} -gt 0 ]]; then
            echo_and_log "The following necessary components have not been installed:${essential_need_install_lib_print[*]}" "${DOING}" "${INSTALL_LOG}"
            echo "${ESSENTIAL_LOST}=${essential_need_install_lib_print[*]}" >> "${TEMP_INI}"
        fi
        if [[ ${#non_essential_need_install_lib[@]} -gt 0 ]]; then
            echo_and_log "The following optional components have not been installed:${non_essential_need_install_lib[*]}" "${DOING}" "${INSTALL_LOG}"
            echo "${NOESSENTIAL_LOST}=${non_essential_need_install_lib[*]}" >> "${TEMP_INI}"
        fi
        #包交互模式
        if [[ "${install_mode}" == "agent" ]] || [[ "${install_mode}" == "web" ]]; then
            install_essential_fail_array=()
            # 自动安装必需依赖
            if [[ ${#essential_need_install_lib_print[@]} -gt 0 ]]; then
                local j=0
                for ((i=0;i<${#essential_need_install_lib_print[@]};i++))
                do
                    echo_and_log "${installer_tool} install -y ${essential_need_install_lib_print[$i]}" "${DOING}" "${INSTALL_LOG}"
                    ${installer_tool} install -y "${essential_need_install_lib_print[$i]}" >> "${INSTALL_LOG}" 2>&1
                    if [[ "$?" -eq 0 ]] ; then
                        echo_and_log "${essential_need_install_lib_print[$i]} Successful installation!" "${SUCCESS}" "${INSTALL_LOG}"
                    else
                        echo_and_log "Install ${essential_need_install_lib_print[$i]} failed" "${ERROR}" "${INSTALL_LOG}"
                        install_essential_fail_array[$j]=${essential_need_install_lib_print[$i]}
                        let j++
                    fi
                done
                sed -i '/^essential_lost=/d' "${TEMP_INI}"
                if [[ -n "${install_essential_fail_array}" ]]; then
                    echo_and_log "Failed to install the necessary components. Please manually install the following components: ${install_essential_fail_array[*]} " "${ERROR}" "${INSTALL_LOG}"
                    # 更新失败的依赖信息
                    echo "${ESSENTIAL_LOST}=${install_essential_fail_array[*]}" >> "${TEMP_INI}"
                    echo_and_log "Suggestion:" "${ERROR}" "${INSTALL_LOG}"
                    echo_and_log "1. Check whether the yum or apt source is correctly configured" "${ERROR}" "${INSTALL_LOG}"
                    echo_and_log "2. Manually install the missing components." "${ERROR}" "${INSTALL_LOG}"
                    echo_and_log "Check install Dependent Packages Failed" "${ERROR}" "${INSTALL_LOG}"
                    exit "${ERROR}"
                fi
                echo_and_log "Check install Dependent Packages Success" "${SUCCESS}" "${INSTALL_LOG}"
            fi
        fi
    fi
}

function get_lib_path()
{
    local lib_array=("/lib" "/usr/lib" "/usr/lib64/")
    local lib_path_array=("/etc/profile" )

    for lib_str in ${lib_path_array[@]}
    do
        if [[ -f "${lib_str}" ]]; then
            local lib_path_str=$(grep -w "LD_LIBRARY_PATH=" "${lib_str}" | tail -1 | awk -F= '{if($2 !="") print $2}')
            if [[ -n "${lib_path_str}" ]]; then
                local array=(${lib_path_str//:/ })
                lib_path=(${lib_path[@]} ${array[@]})
            fi
        fi
    done

    if [[ -f "/etc/ld.so.conf" ]]; then
        for line in $(cat "/etc/ld.so.conf")
        do
            lib_path=(${lib_path[@]} ${line})
        done
    fi
    lib_path=(${lib_path[@]} ${lib_array[@]})
}

function main() {
    if [[ ${#*} -gt 2 ]]; then
        echo "Invalid parameters number, must be less or equal two"
        exit "${ERROR}"
    fi
    # 预检查时为包解压路径：/home/devkit_package/DevKit-All-24.0.RC1-Linux-Kunpeng/DevKit-All-24.0.RC1-Linux-Kunpeng/
    # 正式安装时为框架安装路径：/opt/DevKit
    INSTALL_PATH="$1"
    install_mode="$2"
    init_variable
    if [[ "${install_mode}" == "agent" ]] || [[ "${install_mode}" == "web" ]]; then
        check_install_space
    fi
    check_os_type
    package_all=("pre" "essential" "non_essential")
    lib_check
    exit "${ENV_RESULT}"
}

main $@
