#!/bin/bash
#Description: plugin agent upgrade entry
#Copyright: Copyright © Huawei Technologies Co., Ltd. 2023. All rights reserved.
#History: 2023-11-07 created

CURRENT_DIR=$(cd $(dirname $0); pwd)
source "${CURRENT_DIR}/utils.sh"
SUCCESS=0
ERROR=1

# /opt
ROOT_PATH=$1
# ssh or web
UPGRADE_MODE=$2
# DevKit-Sys-Diagnosis-24.0.RC2-Linux-Kunpeng
UPGRADE_PACKAGE_NAME=$3
PLUGIN_NAME="sys_diagnosis"
# 插件状态
INSTALLED=3
UPGRADE_DOING=4
UPGRADE_SUCCESS=10
UPGRADE_FAILED=9
UPGRADE_DAMAGE=6

UPGRADE_TASK_DATABASE_PY_SCRIPT="${CURRENT_DIR}/script/upgrade_task_database.py"
UPGRADE_TASK_DATABASE_PYC_SCRIPT="${CURRENT_DIR}/script/upgrade_task_database.pyc"
# 默认需要回退，不需要回退的exit处需要设置ROLLBACK_FLAG="FALSE"
ROLLBACK_FLAG="TRUE"
trap "exit_clean" EXIT

function exit_clean() {
    exit_code="$?"
    if [[ ${exit_code} -ne 0 ]]; then
        if [[ ${ROLLBACK_FLAG} == "TRUE" ]]; then
            ROLLBACK_FLAG="FALSE"
            roll_back
            exit "${ERROR}"
        fi
    fi
    if [[ -n "${BACKUP_PATH}" ]] && [[ -d "${BACKUP_PATH}" ]] && [[ ! -L "${BACKUP_PATH}" ]]; then
        rm -rf "${BACKUP_PATH}"
    fi
}

function check_param() {
    if [[ "${#*}" -gt "3" ]]; then
        echo "Invalid parameters number"
        ROLLBACK_FLAG="FALSE"
        exit "${ERROR}"
    fi
    if [[ ! -d "${ROOT_PATH}" ]]; then
        echo "Invalid installation path: ${ROOT_PATH}"
        ROLLBACK_FLAG="FALSE"
        exit "${ERROR}"
    fi
    if [[ "${UPGRADE_MODE}" != "ssh" ]] && [[ "${UPGRADE_MODE}" != "web" ]]; then
        echo "Invalid upgrade mode: ${UPGRADE_MODE}"
        ROLLBACK_FLAG="FALSE"
        exit "${ERROR}"
    fi
}

# 判断安装系统类型 是否为docker 或者其他类型容器
function check_system_type() {
    system_is_container=0
    # 检测是否为容器，是容器返回1，否则返回0（当前工具支持docker和lxc容器）
    if [[ -f /.dockerenv ]]; then
        system_is_container=1
        return
    fi
    # docker里没有systemd
    which systemd-detect-virt >/dev/null 2>&1
    if [[ "$?" -eq 0 ]]; then
        # 检测是否为容器
        container=$(systemd-detect-virt -c)
        if [[ "$container" == "none" ]]; then
            return
        fi
        system_is_container=1
        return
    fi
}

# 初始化需要用到的全局变量
function init_var() {
    DEVKIT_BACKUP="${ROOT_PATH}/DevKitBackup/"
    INSTALL_PATH="${ROOT_PATH}/DevKit/"
    TOOLS_PATH="${INSTALL_PATH}/tools/"
    UPGRADE_LOG="${INSTALL_PATH}/logs/sys_diagnosis_upgrade.log"
    if [[ -f "${TOOLS_PATH}/log.sh" ]]; then
        source "${TOOLS_PATH}/log.sh"
        create_log "${UPGRADE_LOG}"
    else
        echo "Could not initialize log file"
        ROLLBACK_FLAG="FALSE"
        exit "${ERROR}"
    fi
    local sys_diagnosis_const="${INSTALL_PATH}/devkitplugins/${PLUGIN_NAME}/deploy/const.sh"
    if [[ -f "${sys_diagnosis_const}" ]]; then
        # 获取老版本需要备份的提权文件
        OLD_SUDO_FILES_LIST=$(grep SUDO_FILES_LIST= "${sys_diagnosis_const}")
        OLD_SUDO_FILES_LIST=$(echo "${OLD_SUDO_FILES_LIST}" | tr -d '"')
        OLD_SUDO_FILES_LIST=(${OLD_SUDO_FILES_LIST:17:${#OLD_SUDO_FILES_LIST}-18})
    else
        echo_and_log "The original plug-in script file cannot be loaded" "${ERROR}" "${UPGRADE_LOG}"
        ROLLBACK_FLAG="FALSE"
        exit "${ERROR}"
    fi
    SUDO_CONF="/etc/sudoers.d/"
    IPERF3_SUDO="${DEVKIT_SUDO_TOOLS}/iperf3/"
    INSTALL_FRAMEWORK_PATH="${INSTALL_PATH}/devkitframework/"
    INSTALL_PLUGINS_PATH="${INSTALL_PATH}/devkitplugins/"
    PLUGIN_INSTALL_BACKEND_PATH="${INSTALL_PLUGINS_PATH}/${PLUGIN_NAME}/"
    PLUGIN_INSTALL_FRONTEND_PATH="${INSTALL_PATH}/tools/nginx/html/devkit/plugins/${PLUGIN_NAME}/"
    PLUGIN_LOG_JSON_PATH="${INSTALL_PATH}/config/log/operation_log/${PLUGIN_NAME}.json"
    PLUGIN_NGINX_CONF_PATH="${INSTALL_PATH}/tools/nginx/conf/location_conf/${PLUGIN_NAME}.conf"
    FRAME_DB_PATH="${INSTALL_FRAMEWORK_PATH}/db.sqlite3"
    PLUGIN_DB_PATH="${INSTALL_PLUGINS_PATH}/sqlite/db.sqlite3"
    PLUGIN_RUNTIME_LOG="${INSTALL_PATH}/config/log/runtime_log/${PLUGIN_NAME}/"

    # 备份目录加入时间后缀，防止误删用户目录
    CURRENT_DATE=$(date "+%Y_%m_%d_%H_%M_%S")
    BACKUP_PATH="${ROOT_PATH}/${PLUGIN_NAME}_backup_${CURRENT_DATE}/"
    BACKUP_BACKEND_PATH="${BACKUP_PATH}/backend/"
    BACKUP_FRONTEND_PATH="${BACKUP_PATH}/frontend/"
    BACKUP_OPERATION_LOG_PATH="${BACKUP_PATH}/operation_log/"
    BACKUP_NGINX_CONF_PATH="${BACKUP_PATH}/nginx_location_conf/"
    BACKUP_DB_PATH="${BACKUP_PATH}/db_file/"
    BACKUP_RUNTIME_LOG_PATH="${BACKUP_PATH}/runtime_log/"
    BACKUP_SUDOS_PATH="${BACKUP_PATH}/sudo_files/"

    PLUGIN_INSTALL_INI_PATH="${PLUGIN_INSTALL_BACKEND_PATH}/config/install.ini"
    # 获取待升级插件的版本号
    old_version="$(grep -w PLUGIN_VERSION "${PLUGIN_INSTALL_INI_PATH}" | awk -F= '{if($2 !="") print $2}')"
}

# 备份恢复
function roll_back() {
    if [[ ! -d "${BACKUP_PATH}" ]]; then
        echo_and_log "No backup is available. Rollback is not required." "${SUCCESS}" "${UPGRADE_LOG}"
        return "${SUCCESS}"
    fi
    echo_and_log "Start rollback." "${DOING}" "${UPGRADE_LOG}"
    restore_miss_match_files
    rm -rf "${PLUGIN_INSTALL_BACKEND_PATH}"
    rm -rf "${PLUGIN_INSTALL_FRONTEND_PATH}"
    declare -A backed_up_array
    backed_up_array["${BACKUP_BACKEND_PATH}/${PLUGIN_NAME}"]="${INSTALL_PLUGINS_PATH}"
    backed_up_array["${BACKUP_FRONTEND_PATH}/${PLUGIN_NAME}"]="${INSTALL_PATH}/tools/nginx/html/devkit/plugins"
    backed_up_array["${BACKUP_DB_PATH}/db.sqlite3"]="${PLUGIN_DB_PATH}"
    backed_up_array["${BACKUP_RUNTIME_LOG_PATH}/${PLUGIN_NAME}"]="${INSTALL_PATH}/config/log/runtime_log"
    backed_up_array["${BACKUP_OPERATION_LOG_PATH}/${PLUGIN_NAME}.json"]="${PLUGIN_LOG_JSON_PATH}"
    backed_up_array["${BACKUP_NGINX_CONF_PATH}/${PLUGIN_NAME}.conf"]="${PLUGIN_NGINX_CONF_PATH}"
    backed_up_array["${BACKUP_SUDOS_PATH}/devkit_sys_diagnosis_sudoers"]="${SUDO_CONF}"
    for src_path in "${!backed_up_array[@]}"; do
        local dst_path=${backed_up_array["${src_path}"]}
        if [[ ! -e "${src_path}" ]]; then
            echo_and_log "Rollback failed. The backup ${src_path} does not exist." "${ERROR}" "${UPGRADE_LOG}"
            update_plugin_status "${UPGRADE_DAMAGE}" "${UPGRADE_DAMAGE}"
            exit "${ERROR}"
        fi
        if [[ -d "${src_path}" ]]; then
            cp -rpf "${src_path}" "${dst_path}"
        else
            cp -pf "${src_path}" "${dst_path}"
        fi
        if [[ $? -ne 0 ]]; then
            echo_and_log "Failed to roll back ${src_path}." "${ERROR}" "${UPGRADE_LOG}"
            update_plugin_status "${UPGRADE_DAMAGE}" "${UPGRADE_DAMAGE}"
            exit "${ERROR}"
        fi
    done

    if [[ "${UPGRADE_MODE}" == "web" ]]; then
        cp -rpf "${BACKUP_SUDOS_PATH}"/* "${DEVKIT_SUDO_TOOLS}"
        if [[ -f "${DEVKIT_SUDO_TOOLS}/devkit_sys_diagnosis_sudoers" ]]; then
            rm -f "${DEVKIT_SUDO_TOOLS}/devkit_sys_diagnosis_sudoers"
        fi
    else
        cd "${DEVKIT_BACKUP}/devkit_tools"
        for ((i=0;i<${#OLD_SUDO_FILES_LIST[@]};i++))
        do
            if [[ -e "${OLD_SUDO_FILES_LIST[$i]}" ]]; then
                cp -fp "${OLD_SUDO_FILES_LIST[$i]}" "${DEVKIT_SUDO_TOOLS}"
            else
                echo_and_log "Failed to roll back ${OLD_SUDO_FILES_LIST[$i]}." "${ERROR}" "${UPGRADE_LOG}"
                update_plugin_status "${UPGRADE_DAMAGE}" "${UPGRADE_DAMAGE}"
                exit "${ERROR}"
            fi
        done
        cd - 1>/dev/null
    fi
    # 设置插件状态
    update_plugin_status "${INSTALLED}" "${INSTALLED}"
    echo_and_log "Rollback finished." "${SUCCESS}" "${UPGRADE_LOG}"
    rm -rf "${BACKUP_PATH}"
}

# 备份旧版本数据
function backup_data() {
    if [[ -L "${BACKUP_PATH}" ]]; then
        echo_and_log "The backup directory has a soft link" "${ERROR}" "${UPGRADE_LOG}"
        ROLLBACK_FLAG="FALSE"
        exit "${ERROR}"
    fi
    if [[ -d "${BACKUP_PATH}" ]]; then
        rm -rf "${BACKUP_PATH}"
    fi
    mkdir "${BACKUP_PATH}"
    mkdir "${BACKUP_BACKEND_PATH}"
    mkdir "${BACKUP_FRONTEND_PATH}"
    mkdir "${BACKUP_OPERATION_LOG_PATH}"
    mkdir "${BACKUP_NGINX_CONF_PATH}"
    mkdir "${BACKUP_DB_PATH}"
    mkdir "${BACKUP_RUNTIME_LOG_PATH}"
    mkdir "${BACKUP_SUDOS_PATH}"
    chmod 700 "${BACKUP_PATH}" -R

    declare -A backup_array
    backup_array["${PLUGIN_INSTALL_BACKEND_PATH}"]="${BACKUP_BACKEND_PATH}"
    backup_array["${PLUGIN_INSTALL_FRONTEND_PATH}"]="${BACKUP_FRONTEND_PATH}"
    backup_array["${PLUGIN_LOG_JSON_PATH}"]="${BACKUP_OPERATION_LOG_PATH}"
    backup_array["${PLUGIN_NGINX_CONF_PATH}"]="${BACKUP_NGINX_CONF_PATH}"
    backup_array["${PLUGIN_DB_PATH}"]="${BACKUP_DB_PATH}"
    backup_array["${PLUGIN_RUNTIME_LOG}"]="${BACKUP_RUNTIME_LOG_PATH}"
    for src_path in "${!backup_array[@]}"; do
        local dst_path=${backup_array["${src_path}"]}
        if [[ ! -e "${src_path}" ]]; then
            rm -rf "${BACKUP_PATH}"
            echo_and_log "The ${src_path} does not exist. Please check the system" "${ERROR}" "${UPGRADE_LOG}"
            ROLLBACK_FLAG="FALSE"
            exit "${ERROR}"
        fi
        cp -rpf "${src_path}" "${dst_path}"
        if [[ $? -ne 0 ]]; then
            rm -rf "${BACKUP_PATH}"
            echo_and_log "Failed to back up ${src_path}. Please check the system" "${ERROR}" "${UPGRADE_LOG}"
            ROLLBACK_FLAG="FALSE"
            exit "${ERROR}"
        fi
    done
    # 备份提权配置文件
    if [[ -f "${SUDO_FILE}" ]]; then
        cp -fp "${SUDO_FILE}" "${BACKUP_SUDOS_PATH}"
    else
        rm -rf "${BACKUP_PATH}"
        echo_and_log "The privilege escalation configuration file ${SUDO_FILE} not exists" "${ERROR}" "${UPGRADE_LOG}"
        ROLLBACK_FLAG="FALSE"
        exit "${ERROR}"
    fi

    # 只有插件单独升级时，才需要备份devkit_tools下的提权文件
    if [[ "${UPGRADE_MODE}" == "web" ]]; then
        # 备份提权脚本
        if [[ -d "${DEVKIT_SUDO_TOOLS}" ]]; then
            ls -l "${IPERF3_SUDO}"
            if [[ -d "${IPERF3_SUDO}" ]]; then
                cp -rfp "${IPERF3_SUDO}" "${BACKUP_SUDOS_PATH}"
            else
                rm -rf "${BACKUP_PATH}"
                echo_and_log "Directory ${IPERF3_SUDO} not exists" "${ERROR}" "${UPGRADE_LOG}"
                ROLLBACK_FLAG="FALSE"
                exit "${ERROR}"
            fi
            cd "${DEVKIT_SUDO_TOOLS}"
            for ((i=0;i<${#OLD_SUDO_FILES_LIST[@]};i++))
            do
                if [[ -e "${OLD_SUDO_FILES_LIST[$i]}" ]]; then
                    cp -fp "${OLD_SUDO_FILES_LIST[$i]}" "${BACKUP_SUDOS_PATH}"
                else
                    rm -rf "${BACKUP_PATH}"
                    echo_and_log "File ${OLD_SUDO_FILES_LIST[$i]} not exists" "${ERROR}" "${UPGRADE_LOG}"
                    ROLLBACK_FLAG="FALSE"
                    exit "${ERROR}"
                fi
            done
            cd - 1>/dev/null
        else
            rm -rf "${BACKUP_PATH}"
            echo_and_log "Directory ${DEVKIT_SUDO_TOOLS} not exists" "${DOING}" "${UPGRADE_LOG}"
            ROLLBACK_FLAG="FALSE"
            exit "${ERROR}"
        fi
    fi
}

# 刷新框架数据库中插件的安装状态 只需要更新升级失败场景 成功场景框架中已更新
# status={6：升级失败，回滚升级前版本失败，3：升级失败，回滚升级前版本成功}
function update_plugin_status(){
    local status=$1
    local progress_status=$2
    local plugin_version
    plugin_version="$(grep -w PLUGIN_VERSION "${PLUGIN_INSTALL_INI_PATH}" | awk -F= '{if($2 !="") print $2}')"
    local update_time
    update_time=$(date '+%Y-%m-%d %H:%M:%S')
    local sqlite3_bin_path
    sqlite3_bin_path=$(which sqlite3)
    "${sqlite3_bin_path}" "${FRAME_DB_PATH}" "update pluginmanage_plugin set status='$status',progress_status='$progress_status',current_version='$plugin_version',update_time='$update_time' where plugin_id='devkit.${PLUGIN_NAME}';"
}

function load_old_db_data() {
    cp -fp "${BACKUP_DB_PATH}/db.sqlite3" "${INSTALL_PLUGINS_PATH}/sqlite"
    if [[ $? -ne 0 ]]; then
        echo_and_log "Failed to restore sqlite3 file." "${ERROR}" "${UPGRADE_LOG}"
        exit "${ERROR}"
    fi
    # 获取备份目录sys_perf下的所有migrations
    local database_pkg_dir="${BACKUP_BACKEND_PATH}/${PLUGIN_NAME}"
    local backup_migrations_dir_list=($(find "${BACKUP_BACKEND_PATH}/" -path '*/sys_diagnosis*/migrations'))
    local i=0
    for ((i=0;i<${#backup_migrations_dir_list[@]};i++))
    do
        backup_migrations_dir="${backup_migrations_dir_list[$i]}"
        # 替换migrations
        app_migrations_path="${backup_migrations_dir:$((${#database_pkg_dir}+1))}"
        local install_app_migrations_path="${PLUGIN_INSTALL_BACKEND_PATH}/${app_migrations_path}"
        if [ ! -d "${install_app_migrations_path}" ];then
            mkdir "${install_app_migrations_path}"
            chmod 700 "${install_app_migrations_path}"
            chown "${DEVKIT_USER}:${DEVKIT_USER}" "${install_app_migrations_path}"
        fi
        cp -rfp "${database_pkg_dir}/${app_migrations_path}"/* "${PLUGIN_INSTALL_BACKEND_PATH}/${app_migrations_path}"
    done
    export LD_LIBRARY_PATH=${DEVKIT_HOME}/libsqlite3:${DEVKIT_HOME}/libssl:${DEVKIT_HOME}/lib
    source "${INSTALL_PATH}/tools/python_venv/bin/activate"
    python3 "${INSTALL_PLUGINS_PATH}/manage.pyc" makemigrations
    make_migrations_code=$?
    python3 "${INSTALL_PLUGINS_PATH}/manage.pyc" migrate
    migrate_code=$?
    deactivate
    unset LD_LIBRARY_PATH
    if [[ "${make_migrations_code}${migrate_code}" != "00" ]]; then
        exit "${ERROR}"
    fi
    echo_and_log "Load database success" "${SUCCESS}" "${UPGRADE_LOG}"
}

# 数据库升级
function upgrade_task_db() {
    local upgrade_task_database_script=${UPGRADE_TASK_DATABASE_PYC_SCRIPT}
    if [[ -f "${UPGRADE_TASK_DATABASE_PY_SCRIPT}" ]]; then
        upgrade_task_database_script=${UPGRADE_TASK_DATABASE_PY_SCRIPT}
    fi
    new_version="$(grep -w PLUGIN_VERSION "${PLUGIN_INSTALL_INI_PATH}" | awk -F= '{if($2 !="") print $2}')"
    export LD_LIBRARY_PATH=${DEVKIT_HOME}/libsqlite3:${DEVKIT_HOME}/libssl:${DEVKIT_HOME}/lib
    "${INSTALL_PATH}/tools/python_venv/bin/python3" "${upgrade_task_database_script}" --currentVersion "${old_version}" --upgradeVersion "${new_version}" --installPath "${INSTALL_PATH}"
    if [[ $? -ne 0 ]]; then
        unset LD_LIBRARY_PATH
        echo_and_log "An exception occurs in the database related to the plug-in upgrade task." "${ERROR}" "${UPGRADE_LOG}"
        exit "${ERROR}"
    fi
    unset LD_LIBRARY_PATH
    echo_and_log "The plug-in database is upgraded successfully." "${SUCCESS}" "${UPGRADE_LOG}"
}

function delete_miss_match_files() {
    local miss_match_files=("${INSTALL_PATH}/devkitplugins/sys_perf/usermanager/runlog_level_init.pyc")
    for miss_match_file in ${miss_match_files[@]}
    do
        if [[ -f "${miss_match_file}" ]]; then
            cp -a "${miss_match_file}" "${miss_match_file}.bak"
            rm -f "${miss_match_file}"
        fi
    done
}

function restore_miss_match_files() {
    local miss_match_files=("${INSTALL_PATH}/devkitplugins/sys_perf/usermanager/runlog_level_init.pyc")
    for miss_match_file in ${miss_match_files[@]}
    do
        if [[ -f "${miss_match_file}.bak" ]]; then
            cp -a "${miss_match_file}.bak" "${miss_match_file}"
            rm -f "${miss_match_file}.bak"
        fi
    done
}

function upgrade_plugin() {
    # 关闭服务
    gunicorn_plugin_ids=$(ps -ef | grep "devkitplugins/gunicorn_plugin.py" | grep -v 'grep' | awk '{print $2}' | xargs)
    if [[ "${gunicorn_plugin_ids}" ]]; then
        kill -9 ${gunicorn_plugin_ids}
    fi
    if [[ ${system_is_container} == 1 ]]; then
        bash "${INSTALL_PATH}/tools/docker_service.sh" "stop" "gunicorn_plugin"
    else
        systemctl stop gunicorn_plugin.service
    fi
    # 删除原始数据，开始安装
    rm -rf "${PLUGIN_INSTALL_BACKEND_PATH}"
    rm -rf "${PLUGIN_INSTALL_FRONTEND_PATH}"
    rm -f "${PLUGIN_LOG_JSON_PATH}"
    rm -f "${PLUGIN_NGINX_CONF_PATH}"
    rm -rf "${PLUGIN_RUNTIME_LOG}"
    rm -f "${PLUGIN_DB_PATH}"
    rm -rf "${IPERF3_SUDO}"
    for ((i=0;i<${#OLD_SUDO_FILES_LIST[@]};i++))
    do
        rm -f "${DEVKIT_SUDO_TOOLS}/${OLD_SUDO_FILES_LIST[$i]}"
    done
    rm -f "${SUDO_FILE}"
    # 24.0.RC1升级到24.0.RC2时，需要去除旧版本的某些文件，否则会出现导包异常，导致gunicorn_plugin.service失败
    delete_miss_match_files
    if [[ "${UPGRADE_MODE}" == "web" ]]; then
        bash "${TOOLS_PATH}/install_devkit_plugin.sh" "${PLUGIN_NAME}" "${UPGRADE_MODE}" "${UPGRADE_PACKAGE_NAME}"
        local install_flag=$?
    else
        bash "${TOOLS_PATH}/install_devkit_plugin.sh" "${PLUGIN_NAME}"
        local install_flag=$?
    fi
    if [[ "$install_flag" -ne 0 ]]; then
        echo_and_log "Upgrade failed during installation of the Kunpeng System Diagnosis of the new version." "${ERROR}" "${UPGRADE_LOG}"
        # 数据回滚
        exit "${ERROR}"
    fi
    restore_miss_match_files
    # 拷贝老的db.sqlite3文件
    load_old_db_data
    upgrade_task_db
    rm -rf "${BACKUP_PATH}"
}

function main(){
    # 限定Arm版本软件包只能运行在鲲鹏Arm平台
    if [[ "${architecture}" == "aarch64" ]]; then
        if ! is_supported_arm; then
          echo -e "\e[1;31m  The selected upgrade package does not comply with the system architecture.\e[0m"
          exit "${ERROR}"
        fi
    fi
    # 入参校验
    check_param "$@"
    check_system_type
    # 初始化路径
    init_var
    # 数据备份
    backup_data
    # 设置插件状态为升级中
    update_plugin_status "${UPGRADE_DOING}" "${UPGRADE_DOING}"
    # 升级插件
    upgrade_plugin
}

main "$@"