#!/bin/bash
#Description: sys_perf install tool
#Copyright: Copyright © Huawei Technologies Co., Ltd. 2023. All rights reserved.
#History: 2023-6-20 created

current_dir=$(cd $(dirname $0); pwd)
source "${current_dir}/const.sh"

# 创建日志文件
function create_log() {
    local log_path=$1
    if [[ -z "${log_path}" ]]; then
          echo "LOG PATH is empty, please check it"
          return "${ERROR}"
    fi
    if [[ ! -f "${log_path}" ]]; then
      touch "${log_path}"
      chmod 600 "${log_path}"
    else
      echo "" > "${log_path}"
    fi
}

# 终端及日志打印:echo_and_log [info] [status] [log]
function echo_and_log()
{
    local info=$1
    local status=$2
    local log=""
    local type="32"
    if [[ $# == 3 ]]; then
        log=$3 # 选填
        if [[ ! -f "${log}" ]]; then
            echo -e "\e[1;31m echo_and_log:${log} is not exist \e[0m"
            exit "${ERROR}"
        fi
    fi

    case "${status}" in
        0)
            type="32";;
        1)
            type="31";;
        2)
            type="33";;
        3)
            type="37";; # 白色
        *)
            echo -e "\e[1;31m status: 0/1/2, input is ${status} \e[0m";;
    esac
    if [[ -z "${log}" ]]; then
        echo -e "\e[1;${type}m  ${info} \e[0m"
    else
        if [[ ! -f "${log}" ]]; then
            echo -e "\e[1;31m echo_and_log:${log} is not exist \e[0m"
            exit "${ERROR}"
        fi
        echo -e "\e[1;${type}m  ${info} \e[0m" | tee -a "${log}"
    fi
}

# 删除用户
function delete_user() {
    local user_name="$1"
    local package_tool="$2"
    local user_home="${USER_HOME}/${user_name}"
    if [[ "${package_tool}" == "yum" ]]; then
            userdel -rf "${user_name}" >/dev/null 2>&1
        elif [[ "${package_tool}" == "zypper" ]]; then
            userdel -rf "${user_name}" >/dev/null 2>&1
        elif [[ "${package_tool}" == "apt" ]] ; then
            rmuser -h >/dev/null 2>&1
            if [[ $? -eq 0 ]]; then
                rmuser "${user_name}" -rf >/dev/null 2>&1
            else
                userdel -rf "${user_name}" >/dev/null 2>&1
            fi
        else
            deluser -q "${user_name}" >/dev/null 2>&1
        fi
        # userdel 删除用户删除mail下的内容导致创建同名用户报异常
        rm -rf  "/var/spool/mail/${user_name}"
        if [[ -d "${user_home}" ]]; then
            rm -rf "${user_home}"
        fi
}

# 删除应用运行用户
function delete_launcher() {
    local os_type=$1
    local user_name="devkitworker1"
    local package_tool=${COMMAND_LIST["${os_type}"]}
    delete_user "${user_name}" "${package_tool}"
}

# 额外创建安全执行app的普通用户devkitworker1
function create_launcher()
{
    local os_type=$1
    local user_name="devkitworker1"
    local user_home="${USER_HOME}/${user_name}"
    local user_group="devkit"
    local package_tool=${COMMAND_LIST["${os_type}"]}
    local user_exist
    user_exist=$(cat ${OS_PASSWD} | grep -w "${user_name}")
    if [[ -n "${user_exist}" ]]; then
        # 先删除
        echo_and_log "User:${user_name} is exist!" "${SUCCESS}"
        delete_user "${user_name}" "${package_tool}"
    fi
    # 再创建
    if [[ "${package_tool}" == "apt" ]] || [[ "${package_tool}" == "zypper" ]]; then
        groupadd "${user_name}"
        useradd "${user_name}" -d "${user_home}" -g "${user_group}" -m -c "devkit_advisor_user"
    elif [[ "${package_tool}" == "yum" ]]; then
        useradd "${user_name}" -g "${user_group}" -c "devkit_advisor_user"
    else
        echo_and_log "Not supported OS type:${os_type}!" "${ERROR}"
        exit "${ERROR}"
    fi
    if [[ "$?" -eq "${SUCCESS}" ]]; then
        chmod 700 "${user_home}"
    else
        if [[ "${os_type}" == "SLESOS" ]]; then
            userdel -f "${user_name}"
            groupdel "${user_name}"
        elif [[ "${package_tool}" == "yum" ]]; then
            userdel -f "${user_name}"
        else
            userdel "${user_name}"
        fi
        echo_and_log "create_launcher:add user failed!" "${ERROR}"
        exit "${ERROR}"
    fi

    usermod "${user_name}" -s /sbin/nologin
    usermod "${user_name}" -G wheel
    passwd -l "${user_name}"
    # 设置launch用户umask值
    if [[ -f "${user_home}/.bash_profile" ]]; then
        env_file="${user_home}/.bash_profile"
    elif [[ -f "${user_home}/.profile" ]]; then
        env_file="${user_home}/.profile"
    else
        echo_and_log "deal_env:env_file is not exist" "${ERROR}"
        exit "${ERROR}"
    fi
    sed -i -e "/umask/d" "${env_file}"
    sed -i "s%HYPER_WORKER_HOME=.*%HYPER_WORKER_HOME=${user_home}%g" "${CONFIG_FILE}"
    echo "umask 077" >> "${env_file}"
}

function copy_sudo_files() {
    local install_path=$1    # /opt/DevKit
    local install_log=$2
    # agent模式则移动文件，否则复制
    local install_mode=$3
    # 复制新目录的提权文件到DEVKIT_TOOLS目录
    local sys_diagnosis_sudo_dir="${install_path}/devkitplugins/sys_diagnosis/components/tool/"
    if [[ -d "${sys_diagnosis_sudo_dir}" ]]; then
        cd "${sys_diagnosis_sudo_dir}"
        for ((i=0;i<${#SUDO_FILES_LIST[@]};i++))
        do
            if [[ -e "${SUDO_FILES_LIST[$i]}" ]]; then
                if [[ "${install_mode}" == "agent" ]]; then
                    mv -f "${SUDO_FILES_LIST[$i]}" "${DEVKIT_SUDO_TOOLS}"
                else
                    cp -f "${SUDO_FILES_LIST[$i]}" "${DEVKIT_SUDO_TOOLS}"
                    chmod 400 "${SUDO_FILES_LIST[$i]}"
                fi
                chmod 500 "${DEVKIT_SUDO_TOOLS}/${SUDO_FILES_LIST[$i]}"
                chown root:root "${DEVKIT_SUDO_TOOLS}/${SUDO_FILES_LIST[$i]}"
                echo_and_log "Copy ${SUDO_FILES_LIST[$i]} to devkit_tools success" "${SUCCESS}" "${install_log}"
            else
                echo_and_log "File ${SUDO_FILES_LIST[$i]} not exists, continue" "${DOING}" "${install_log}"
            fi
        done
        cd - 1>/dev/null
    else
        echo_and_log "Directory ${sys_diagnosis_sudo_dir} not exists, continue" "${DOING}" "${install_log}"
    fi

    echo_and_log "Copy risky command success" "${SUCCESS}" "${install_log}"

    # 复制install.ini到提权目录下
    local install_ini="${USER_HOME}/devkit/devkit_conf/install.ini"
    if [[ -f "${install_ini}" ]]; then
        local sudo_install_ini="/usr/bin/devkit_tools/install.ini"
        if [[ ! -f "${sudo_install_ini}" ]]; then
            cp "${install_ini}" "${sudo_install_ini}"
            chown root:root "${sudo_install_ini}"
            chmod 400 "${sudo_install_ini}"
        fi
    fi
}

function add_devkit_plugin_sudo()
{
    command="$1"
    command_path="$2"
    local install_log="$3"
    local sudo_file="$4"
    # 跳过空提权命令
    if [[ -z "${command}" ]]; then
        return
    fi
    # 跳过不存在的提权命令
    if [[ ! -e "${command}" ]]; then
        echo_and_log  "Not exits command, skip: ${command}" "${DOING}" "${install_log}"
        return
    fi

    if [[ ! -f "${sudo_file}" ]]; then
        touch "${sudo_file}"
    fi
    chmod 440 "${sudo_file}"

    if [[ ! -z "${command_path}" ]]; then
        # 空文件sed 追加不进去 sed 根据行号追加内容
        echo "${DEVKIT_USER}    ALL=(root)      NOPASSWD:""${command_path}" >> "$sudo_file"
    fi
}

function add_sudoers() {
    local install_path=$1
    local install_log=$2
    local ping_path
    ping_path=$(which ping 2>/dev/null)
    local traceroute_path
    traceroute_path=$(which traceroute 2>/dev/null)
    local ibv_rc_pingpong_path
    ibv_rc_pingpong_path=$(which ibv_rc_pingpong 2>/dev/null)
    local ibv_uc_pingpong_path
    ibv_uc_pingpong_path=$(which ibv_uc_pingpong 2>/dev/null)
    local ibv_ud_pingpong_path
    ibv_ud_pingpong_path=$(which ibv_ud_pingpong 2>/dev/null)
    local ib_write_lat_path
    ib_write_lat_path=$(which ib_write_lat 2>/dev/null)
    local ib_write_bw_path
    ib_write_bw_path=$(which ib_write_bw 2>/dev/null)
    local ib_read_lat_path
    ib_read_lat_path=$(which ib_read_lat 2>/dev/null)
    local ib_read_bw_path
    ib_read_bw_path=$(which ib_read_bw 2>/dev/null)
    local ib_send_lat_path
    ib_send_lat_path=$(which ib_send_lat 2>/dev/null)
    local ib_send_bw_path
    ib_send_bw_path=$(which ib_send_bw 2>/dev/null)
    echo_and_log "Add sudoers" "${DOING}" "${install_log}"
    parameter_list=(
        "${DEVKIT_SUDO_TOOLS}/leak_check.sh"
        "${DEVKIT_SUDO_TOOLS}/clean_memdiagnostic.sh"
        "${DEVKIT_SUDO_TOOLS}/fio.sh"
        "${DEVKIT_SUDO_TOOLS}/tcpdump.sh"
        "${DEVKIT_SUDO_TOOLS}/iperf3.sh"
        "${DEVKIT_SUDO_TOOLS}/netflow.sh"
        "${ping_path}"
        "${traceroute_path}"
        "${ibv_rc_pingpong_path}"
        "${ibv_uc_pingpong_path}"
        "${ibv_ud_pingpong_path}"
        "${ib_write_lat_path}"
        "${ib_write_bw_path}"
        "${ib_read_lat_path}"
        "${ib_read_bw_path}"
        "${ib_send_lat_path}"
        "${ib_send_bw_path}"
    )
    parameter_list_len=${#parameter_list[@]}
    for ((i=0;i<${parameter_list_len};i++)); do
        local parameter=${parameter_list[i]}
        local cmd_name
        cmd_name=$(echo ${parameter} | cut -d " " -f 1)
        add_devkit_plugin_sudo "${cmd_name}" "${parameter}" "${install_log}" "${SUDO_FILE}"
    done
}

function create_tmp_dir() {
    local install_path="$1"
    local tmp_dir="${install_path}/devkitplugins/sys_diagnosis/tmpdir/"
    if [[ ! -d "${tmp_dir}" ]]; then
        mkdir "${tmp_dir}"
        chown "${DEVKIT_USER}":"${DEVKIT_USER}" "${tmp_dir}"
        chmod 700 "${tmp_dir}"
    fi
    local devkit_tools_tmp_dir="/usr/bin/devkit_tools/tmpdir/"
    if [[ ! -d "${devkit_tools_tmp_dir}" ]]; then
        mkdir "${devkit_tools_tmp_dir}"
        chown root:root "${devkit_tools_tmp_dir}"
        chmod 700 "${devkit_tools_tmp_dir}"
    fi
}

function delete_sudo_tools() {
    local LOG=$1
    echo_and_log "Delete sudo tools" "${DOING}" "${LOG}"
    global_sudo_tools=("kill_process.sh" "perf" "set_env.sh" "pidstat.sh" "perf_archive" "directory_retrieval" "chmod_binary" "chown_binary" "rm_binary" "cp_binary" "cat_binary" "echo_binary" "tee_binary" "tar_binary" "mv_binary" "locale_binary" "dmidecode_binary")
    cd ${DEVKIT_SUDO_TOOLS}
    if [[ -f "${SUDO_FILE}" ]]; then
        for sudo_tool in $(grep NOPASSWD:/usr/bin/devkit_tools/ "${SUDO_FILE}" | awk -F/usr/bin/devkit_tools/ '{print $2}' | awk -F' ' '{print $1}')
        do
            if [[ -e "${sudo_tool}" ]] && [[ -f "${sudo_tool}" ]]; then
                if [[ ${global_sudo_tools[@]} =~ ${sudo_tool} ]]; then
                    continue
                fi
                rm -f "${sudo_tool}"
            fi
        done
        rm -f "${SUDO_FILE}"
    fi
    for ((i=0; i<${#SUDO_FILES_LIST[@]}; i++))
    do
        file_path="${DEVKIT_SUDO_TOOLS}/${SUDO_FILES_LIST[$i]}"
        if [[ -e "${file_path}" ]]; then
            rm -rf "${file_path}"
        fi
    done

    # 不在提权配置文件中，但是在/usr/bin/devkit_tools/目录下的文件同样需要删除
    local other_tools=("iperf3" "trace_net_flow")
    for ((i=0;i<${#other_tools[@]};i++))
    do
        if [[ -d "${other_tools[$i]}" ]]; then
            rm -rf "${other_tools[$i]}"
        elif [[ -f "${other_tools[$i]}" ]]; then
            rm -f "${other_tools[$i]}"
        else
            echo_and_log "File ${other_tools[$i]} not exists, continue" "${DOING}"
        fi
    done
    rm -rf trace_net_flow_kern.o.*
    echo_and_log "Delete sudo tools success" "${SUCCESS}" "${LOG}"
    cd - 1>/dev/null
}

function safely_delete_file() {
    local file_path="$1"
    local install_path="$2"
    local libssl_path="${USER_HOME}/devkit/libssl/"
    local openssl_path="${install_path}/tools/openssl_contents/openssl_bin/openssl"
    local size=0
    if [[ -f "${file_path}" ]]; then
        size=$(du -k "${file_path}" | awk '{if($1 !="") print $1}')
        size=$((size*2))
    else
        echo_and_log "File ${file_path} not exists, continue" "${DOING}"
        return
    fi
    if [[ ${size} -gt 1000 ]] || [[ ${size} -lt 1 ]]; then
        echo_and_log "File ${file_path} is over range, continue" "${DOING}"
        return
    fi
    # 0覆写
    dd if=/dev/zero of=${file_path} bs=512 count=${size} >> /dev/null 2>&1
    # 1覆写
    cat /dev/zero | tr "0" "377" | dd of=${file_path} count=${size} >> /dev/null 2>&1
    # 随机数覆写
    LD_LIBRARY_PATH=${libssl_path} ${openssl_path} rand 512 | dd of=${file_path} count=${size} >> /dev/null 2>&1
    rm -f "${file_path}"
}

function safely_delete_folder() {
    local folder_path="$1"
    local install_path="$2"
    for file_path in $(ls ${folder_path})
    do
        if [[ -d "${folder_path}/${file_path}" ]]; then
            safely_delete_folder "${folder_path}/${file_path}" "${install_path}" >> /dev/null 2>&1
        else
            safely_delete_file "${folder_path}/${file_path}" "${install_path}" >> /dev/null 2>&1
        fi
    done
    rm -rf "${folder_path}"
}

function install_third_app() {
    local install_log="$1"
    cd ${current_dir}
    tar -zxf ${IPERF_PACKAGE} >> /dev/null 2>&1
    if [[ "$?" -ne "${SUCCESS}" ]]; then
        echo_and_log "Decompress ${IPERF_PACKAGE} failed" "${ERROR}" "${install_log}"
        exit "${ERROR}"
    fi
    pushd ${IPERF} >> /dev/null 2>&1
    if [[ "$?" -eq "${SUCCESS}" ]]; then
        if [[ ! -d "${DEVKIT_SUDO_TOOLS}" ]]; then
            mkdir "${DEVKIT_SUDO_TOOLS}"
            chmod 700 "${DEVKIT_SUDO_TOOLS}"
        fi
        ./configure CC=$(which gcc) --prefix=${DEVKIT_SUDO_TOOLS}/iperf3 CFLAGS="-fstack-protector-all" LDFLAGS="-Wl,-z,relro,-z,now -z,noexecstack" >> /dev/null 2>&1
    else
        echo_and_log "cd iperf dir failed" "${ERROR}" "${install_log}"
        exit "${ERROR}"
    fi
    if [[ "$?" -ne "${SUCCESS}" ]]; then
        echo_and_log "Configure iperf Failed" "${ERROR}" "${install_log}"
        check_gcc
        exit "${ERROR}"
    fi
    make -j4 >> /dev/null 2>&1
    if [[ "$?" -ne "${SUCCESS}" ]]; then
        echo_and_log "Make iperf3 Failed" "${ERROR}" "${install_log}"
        check_gcc
        exit "${ERROR}"
    fi
    make install >> /dev/null 2>&1
    if [[ "$?" -ne "${SUCCESS}" ]]; then
        echo_and_log "Make install iperf3 Failed" "${ERROR}" "${install_log}"
        check_gcc
        exit "${ERROR}"
    fi
    chmod -R g-rwx,o-rwx "${DEVKIT_SUDO_TOOLS}/iperf3"
    strip "${DEVKIT_SUDO_TOOLS}/iperf3/lib/libiperf.so"
    strip "${DEVKIT_SUDO_TOOLS}/iperf3/bin/iperf3"
    echo_and_log "Install iperf3 Success" "${SUCCESS}" "${install_log}"
    popd >> /dev/null 2>&1
    # 删除解压后的目录
    if [[ -d iperf ]]; then
        rm -rf iperf
    fi
    if [[ -f "${NET_FLOW_PACKAGE}" ]]; then
        if [[ -f "${DEVKIT_SUDO_TOOLS}/trace_net_flow" ]]; then
            rm -rf "${DEVKIT_SUDO_TOOLS}/trace_net_flow"
        fi
        unzip -oq "${NET_FLOW_PACKAGE}" -d "${DEVKIT_SUDO_TOOLS}"
        if [[ $? -eq "${SUCCESS}" ]]; then
            echo_and_log "Install trace_net_flow Success" "${SUCCESS}" "${install_log}"
        else
            echo_and_log "Install trace_net_flow Failed" "${ERROR}" "${install_log}"
        fi
        if [[ -d "${DEVKIT_SUDO_TOOLS}/trace_net_flow" ]]; then
            mv ${DEVKIT_SUDO_TOOLS}/trace_net_flow "${DEVKIT_SUDO_TOOLS}/trace_net"
            mv ${DEVKIT_SUDO_TOOLS}/trace_net/t* "${DEVKIT_SUDO_TOOLS}"
            rm -rf "${DEVKIT_SUDO_TOOLS}/trace_net"
        fi
    fi
}

# 编译失败检查gcc或者ld是否被篡改
function check_gcc() {
    local gcc_path
    gcc_path=$(which gcc)
    local ld_path
    ld_path=$(which ld)
    echo "gcc path: ${gcc_path}"
    echo "ld path: ${ld_path}"
    ${gcc_path} --version | grep "gcc "
    if [[ $? -ne 0 ]]; then
        echo_and_log "GCC(${gcc_path}) is abnormal, please check" "${ERROR}"
        return "${ERROR}"
    fi
    ${ld_path} --version | grep "GNU ld"
    if [[ $? -ne 0 ]]; then
        echo_and_log "ld(${ld_path}) is abnormal, please check" "${ERROR}"
        return "${ERROR}"
    fi
    if [[ "${LD}" != "${ld_path}" ]]; then
        echo_and_log "The LD environment variable (${LD}) is abnormal, please check" "${ERROR}"
        return "${ERROR}"
    fi
}

# 获取系统磁盘空间，检查磁盘空间是否满足安装
function disk_space_check()
{
    local install_path=$1
    local disk_space_need=$2
    local install_path_parent
    install_path_parent=$(dirname "${install_path}")
    # 获取磁盘的百分之十的大小，与disk_space_need比较
    local total_size
    total_size=$(df "${install_path_parent}" | awk 'NR==2' | awk '{print $2}')
    local percent_size=$((total_size / 10))
    if [[ ${percent_size} -gt ${TOOL_USE_SPACE} ]]; then
        disk_space_need=$((TOOL_USE_SPACE + disk_space_need))
    else
        disk_space_need=$((percent_size + disk_space_need))
    fi

    local available_space=$(df "$install_path_parent" | awk 'NR==2' | awk '{print $4}')
    if [[ ${available_space} -lt ${disk_space_need} ]]; then
        print_disk_space_need=$((disk_space_need / 1000 / 1000))
        echo_and_log "Insufficient drive space. At least ${print_disk_space_need} GB free space is required. Release drive space and try again." "${ERROR}"
        return "${ERROR}"
    fi
    return "${SUCCESS}"
}

function is_supported_arm() {
    current_cpu_id="unknown"
    if [[ -f "${CPU_ID_FILE}" ]]; then
        current_cpu_id=$(cat "${CPU_ID_FILE}")
    fi
    for value in "${SUPPORTED_ARM[@]}"
    do
        if [[ "${current_cpu_id}" == "${value}" ]]; then
            return "${SUCCESS}"
        fi
    done
    return "${ERROR}"
}

function check_kunpeng() {
    local install_log="$1"
    if [[ "${architecture}" != "aarch64" ]]; then
        echo_and_log "\nFailed to install the System Diagnosis. x86 platform not supported." "${ERROR}" "${install_log}"
        exit "${ERROR}"
    else
        if ! is_supported_arm; then
            echo_and_log "\nFailed to install the System Diagnosis. Only the Kunpeng platform is supported." "${ERROR}" "${install_log}"
            exit "${ERROR}"
        fi
    fi
}
